-- ***** BEGIN GPL LICENSE BLOCK *****
--
-- Script copyright (C) Robert "Derobrash" Ashton
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software Foundation,
-- Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- ***** END GPL LICENCE BLOCK *****
-- --------------------------------------------------------------------------

--Version History--

--	Version 0.2
--Initial Release
--	Version 0.3
--Fixed method of splitting meshes up by material causing corrupt model exports


rollout MeshRollout "Dero's Movies Mesh Tool" width:268 height:328
(
	button btnImport "Import Mesh" pos:[50,33] width:150 height:98
	button btnExport "Export Mesh" pos:[50,141] width:150 height:98
	
	fn ReadFixedString stream fixedLen=
	(
		local str = ""
		for i = 1 to fixedLen do
		(
			local x = ReadByte stream #unsigned
			if x != 0 then
			(
				str += bit.intAsChar x
			)
		)		
		str
	)
	
	fn GetNodeCount o=
	(
		local Count = 0
		
		for c in o.Children do
		(
			if c.Renderable == true then
			(
				Count = Count + 1
			)
			Count = Count + GetNodecount c
		)
		Count
	)
	
	fn SkipBytes stream count=
	(
		for i = 1 to count do
		(
			ReadByte stream #unsigned 
		)	
	)	
	
	fn GetDecodedValue stream MinVal MaxVal=
	(
		local val = ReadShort stream #unsigned	
		local normalized = (MaxVal - MinVal) / 65535.0
		local result =  MinVal + (val * normalized) 

		result		
	)
	
	fn EncodeValue value mini maxi=
	(
		local diff = maxi - mini
		local scalevalue = 65535.0 / diff				
		local scaled = (value - mini) * scalevalue

		scaled	
	)	
			
	struct TMMaterial
	(
		MapOne,
		MapTwo,
		MapThree,
		MapFour,
		Bytes
	)
	
	struct Vector3
	(
		x,y,z
	)

	struct TMSubset
	(	
		MatID,
		MinPos,
		MaxPos,
		MinTex,
		MaxTex,
		Vertices,
		Normals,
		TexCoords,	
		Indices
	)		
		
	struct TMMesh
	(
	
		Magic,
		TextureCount,
		MaterialCount,
		GroupCount,
		HasBones,
		UnknownOne,
		AnchorCount,
		HasBinding,
		UnknownTwo,
		FlagsOne,
		FlagsTwo,
		HasDeforms,
		TransformCount,
		SubmeshCount,
		RoomCount,		
		
		TextureFilenames,
		Materials,
		MaxMaterials,
		
		
		fn LoadControlMesh stream name=
		(	
		
			local Mat = StandardMaterial()
			-- Temporary Geometry Data
			local VertexCount = ReadLong stream #unsigned
			local FaceCount = ReadLong stream #unsigned
			local Vertices = #()
			local Indices = #()
			local a,b,c
			
			for i = 1 to VertexCount do
			(
				-- Load Vertices
				a = ReadFloat stream
				b = ReadFloat stream
				c = ReadFloat stream
				
				append Vertices [ a, b, c ]				
			)

			for i = 1 to FaceCount do
			(
				-- Load Indices
				a = ReadShort stream #unsigned
				b = ReadShort stream #unsigned
				c = ReadShort stream #unsigned

				append Indices ( [ a, b, c] + [1,1,1] )
			)
			
			if ( bit.and FaceCount 1) == 1 then
			(
				SkipBytes stream 2
			)


			-- We have the Geometry, make a new mesh for it
			ControlMesh = mesh name:(name) vertices:Vertices faces:Indices\
			material:Mat
			
			update ControlMesh
			gc()
			
		),
		
		fn LoadSubset stream profilestream=
		(
			-- Load Geometry into 3ds!
		
			local ID = ReadLong stream #unsigned
			local FaceCount = ReadLong stream #unsigned
			local VertexCount = ReadLong stream #unsigned
			local Flags = #()
			local Vertices = #()
			local TexCoords = #()
			local Normals = #()
			local Indices = #()
			
			print FaceCount
			print VertexCount
			
			for i = 1 to 4 do
			(
				Flags[i] = ReadByte stream #unsigned
			)			
			
			-- Skip some of the optional stuff
			
			if ( bit.and Flags[1] 16 ) == 16 then
			(
				SkipBytes stream 4
			)
			
			if ( bit.and Flags[1] 32 ) == 32 then
			(
				SkipBytes stream 4
			)
			if ( bit.and Flags[1] 64 ) == 64 then
			(
				SkipBytes stream 4
			)			
			
			-- Read in Modifiers
			
			local MinPos = Vector3()
			local MaxPos = Vector3()
			local MinTex = Vector3()
			local MaxTex = Vector3()
			
			
			-- Read Min
			MinPos.x = ReadFloat stream
			MinPos.y = ReadFloat stream
			MinPos.z = ReadFloat stream
			
			-- Read Max
			MaxPos.x = ReadFloat stream
			MaxPos.y = ReadFloat stream
			MaxPos.z = ReadFloat stream
			
			-- Read Min Tex
			
			MinTex.x = ReadFloat stream
			MinTex.y = ReadFloat stream

			-- Read Max Tex
	
			MaxTex.x = ReadFloat stream
			MaxTex.y = ReadFloat stream
			
	
			-- Skip Unknown

			SkipBytes stream 16
			
			
			-- WILL ALWAYS BE WRITTEN AS ZERO
			local SwitchCount = ReadLong stream #unsigned
			
			local a,b,c
			
			-- Read in the Indices
			
			for i = 1 to FaceCount do
			(
				a = ReadShort stream #unsigned
				b = ReadShort stream #unsigned
				c = ReadShort stream #unsigned
				
				append Indices ([a,b,c] + [1,1,1])							
			)
			
			
			if (bit.and FaceCount 1) == 1 then
			(
				-- Padding
				SkipBytes stream 2
			)
			
			for i = 1 to VertexCount do
			(
			
				-- Read in all the Geometry and scale it
				a = GetDecodedValue stream MinPos.x MaxPos.x
				b = GetDecodedValue stream MinPos.y MaxPos.y
				c = GetDecodedValue stream MinPos.z MaxPos.z
				
				append Vertices [a,b,c]
				
				-- Read in Normals. NB: Should be normalizing these
				a = GetDecodedValue stream -1.0 1.0
				b = GetDecodedValue stream -1.0 1.0
				c = GetDecodedValue stream -1.0 1.0
				
				append Normals [a,b,c]
				
				-- Flip the V coords, dds is upside down
				a = GetDecodedValue stream MinTex.x MaxTex.x
				b = GetDecodedValue stream (1.0 - MinTex.y) (1.0 - MaxTex.y)
				
				append TexCoords [a,b, 1.0]			
			)
			
			
			-- Add this geometry to the environment			
			
		    SubMesh = mesh name:("UnnamedYet") vertices: Vertices faces:Indices\
				material:MaxMaterials[ ID + 1 ]				
				
			-- Set the number of texture coords
			SetNumTVerts SubMesh VertexCount
			
			-- Build the faces
			BuildTVFaces SubMesh false		
			
			
			-- Add the texture coords
			for i = 1 to VertexCount do
			(
				SetTVert SubMesh i TexCoords[i]
				SetNormal SubMesh i Normals[i]
			)
			
			-- Texture Coords use the same indexing
			-- As the Vertices do
			for i = 1 to FaceCount do
			(
				SetTVFace SubMesh i (GetFace SubMesh i)
			)			
			
			update SubMesh			
			
			-- Skip some of the other stuff we don't know about yet
			
			if Flags[2] != 0 then
			(
				for i = 1 to VertexCount do
				(
					SkipBytes stream 4
				)			
			)
			
			-- Skip some more stuff
			
			if ( bit.and Flags[1] 1 ) == 1 then
			(
				for i = 1 to VertexCount do
				(
					SkipBytes stream 20
				)			
			)
			
			-- And the final optional
			
			for i = 1 to SwitchCount do
			(
				SkipBytes stream 8
			)
			
			SubMesh
			
		),		
		
				
		fn LoadGroup stream profilestream=
		(
			-- Temporary Geometry Data
			local Count = ReadByte stream #unsigned
			local Flags = #()
			
			for i = 1 to 3 do
			(
				Flags[i] = ReadByte stream #unsigned
			)
			
			SkipBytes stream 48
			
			if (bit.and Flags[3] 16) == 16 then
			(
				SkipBytes stream 4
			)			
			
			-- Load Subsets into 3ds
			local GroupMeshes = #()
			for i = 1 to Count do
			(
				GroupMeshes[i] = LoadSubset stream profilestream		
				
				if i >= 2 then
				(
					Attach GroupMeshes[1] GroupMeshes[i]
				)				
			)
			
			update GroupMeshes[1]		
		
		),
		
		fn GetSubset stream ID node=
		(
			local Subset = TMSubset()
			local msh = SnapshotAsMesh node
			
			Subset.MatID = ID
			Subset.MinPos = [99999,99999,99999]
			Subset.MaxPos = [-99999,-99999,-99999]
			Subset.MinTex = [99999,99999]
			Subset.MaxTex = [-99999, -99999]
			
			local CurrentPoint					
		
			Subset.Vertices = #()
			Subset.TexCoords = #()
			Subset.Normals = #()
			Subset.Indices = #()
				
			local CurrentFace
			
			local CurrentIndex = 0
			
			
			local sceneMat = MaxMaterials[ID]
			local faceMat
			
			for i = 1 to msh.NumFaces do
			(			
				
				if classof node.Material == MultiMaterial then
				(
					faceMat = node.Material.MaterialList[ GetFaceMatID msh i ]
				)
				else
				(
					faceMat = node.Material
				)				
				
				--if this is the right material				
				if faceMat == sceneMat then
				(				
					CurrentFace = GetFace msh i
					
					for j = 1 to 3 do
					(
						local CurrentVert = GetVert msh CurrentFace[j]
						append Subset.Vertices CurrentVert
						
						for k = 1 to 3 do
						(							
							--check min / max against these vertices
							if(CurrentVert[k] > Subset.MaxPos[k]) then
							(
								Subset.MaxPos[k] = CurrentVert[k]
							)
							if(CurrentVert[k] < Subset.MinPos[k]) then
							(
								Subset.MinPos[k] = CurrentVert[k]
							)	
						)	
						
						append Subset.Normals (GetNormal msh CurrentFace[j])
						append Subset.Indices CurrentIndex
						CurrentIndex = CurrentIndex + 1
					)
					
					CurrentFace = GetTVFace msh i					
					
					for j = 1 to 3 do
					(
						local CurrentCoord = GetTVert msh CurrentFace[j]
						
						append Subset.TexCoords CurrentCoord
						--check min/max against these texcoords
						for k = 1 to 2 do
						(						
							if(CurrentCoord[k] > Subset.MaxTex[k]) then
							(
								Subset.MaxTex[k] = CurrentCoord[k]
							)
							if(CurrentCoord[k] < Subset.MinTex[k]) then
							(
								Subset.MinTex[k] = CurrentCoord[k]
							)
						)						
						
					)
				)													
			)

			Subset			
			
		),
		
		
		fn WriteSubset stream subset =
		(							
			local NumFaces = subset.Indices.Count / 3
			-- Write vital statistics
			WriteLong stream (subset.MatID-1) #unsigned
			WriteLong stream NumFaces #unsigned
			WriteLong stream subset.Vertices.Count #unsigned
									
			-- Write default flags
			WriteLong stream 112

			-- Write some default values : HACK
				
			WriteLong stream ( random 1 320000 ) #unsigned
			WriteLong stream 4294967295 #unsigned
			WriteLong stream 0
	
			for i = 1 to 3 do
			(
				WriteFloat stream subset.MinPos[i]
			)
				
			for i = 1 to 3 do
			(
				WriteFloat stream subset.MaxPos[i]
			)	

			for i = 1 to 2 do
			(
				WriteFloat stream subset.MinTex[i]
			)	
				
			for i = 1 to 2 do
			(
				WriteFloat stream subset.MaxTex[i]
			)
				
			for i = 1 to 20 do
			(
				WriteByte stream 0
			)

			for i = 1 to subset.Indices.Count do
			(
				WriteShort stream (subset.Indices[i]) #unsigned
			)		
								
			if (bit.and NumFaces 1) == 1 then
			(
				WriteShort stream 0
			)
				
			for i = 1 to subset.Vertices.Count do
			(										
					-- Write scaled positional data
				for j = 1 to 3 do
				(
					WriteShort stream ( EncodeValue subset.Vertices[i][j] subset.MinPos[j] subset.MaxPos[j] ) #unsigned
				)					
	
				for j = 1 to 3 do
				(
					WriteShort stream ( EncodeValue subset.Normals[i][j] -1.0 1.0 ) #unsigned
				)
		
				WriteShort stream ( EncodeValue subset.TexCoords[i][1] (subset.MinTex[1]) (subset.MaxTex[1]) ) #unsigned
				WriteShort stream ( EncodeValue subset.TexCoords[i][2] (1.0 - subset.MinTex[2]) (1.0 - subset.MaxTex[2]) ) #unsigned						
				
			)							
		),
		
		fn WriteNode stream o=
		(
		-- Now comes the meat		
			if o.renderable == true then
			(
		
				local SubsetBuffer = #()
				local SubsetCount = 0
				
				for i = 1 to Materials.Count do
				(
					local Tempsubset		
			
					TempSubset = GetSubset stream i o	
					
					if TempSubset.Vertices.Count > 0 then
					(
						SubsetBuffer[SubsetCount + 1] = TempSubset
						SubsetCount = SubsetCount + 1	
					)		
				)						
				
				-- Write Mesh Count
				WriteByte stream SubsetCount #unsigned
				-- Write 3 0 Bytes ( Flags )
				WriteByte stream 0 
				WriteByte stream 0
				WriteByte stream 16
						
				-- Write 3x3 identity + origin as center ( hacky )
				for i = 1 to 3 do
				(
					WriteFloat stream 1.0
					WriteFloat stream 0.0
					WriteFloat stream 0.0
					WriteFloat stream 0.0
				)		
			
				-- Just do it, it may be optional but
				-- otherwise it just screws up
				WriteLong stream 0
				
				--OK, this is where it gets interesting
				--Write list of subsets here
				for i = 1 to SubsetBuffer.Count do
				(	
					WriteSubset stream SubsetBuffer[i]						
				)		
				
			)			
			
			--Do the same for all children
			for c in o.Children do
			(
				WriteNode stream c
			)			
		),
		
		fn SaveMesh filename=
		(				
			local stream = fopen filename "wb"			
				
			TextureFilenames = #()
			Materials = #()
			MaterialCount = 0
			MaxMaterials = #()

			for i = 1 to sceneMaterials.Count do
			(
			if (GetNumSubMtls sceneMaterials[i]) == 0 then
			(
			   print "Material Found"
			   MaterialCount = MaterialCount + 1
			   
			   -- Keep reference to this material as we will be using it later
			   MaxMaterials[MaterialCount]  = sceneMaterials[i]		
	
						Materials[MaterialCount] = TMMaterial()
									
						--Set Defaults
						Materials[MaterialCount].Bytes = #()	
						Materials[MaterialCount].MapOne = -1	
						Materials[MaterialCount].MapTwo = -1
						Materials[MaterialCount].MapThree = -1
						Materials[MaterialCount].MapFour = -1
						Materials[MaterialCount].Bytes[1] = 0
						Materials[MaterialCount].Bytes[2] = 1
						Materials[MaterialCount].Bytes[3] = 1
						Materials[MaterialCount].Bytes[4] = 0
						Materials[MaterialCount].Bytes[5] = 0
						Materials[MaterialCount].Bytes[6] =0
						Materials[MaterialCount].Bytes[7] = 0
						Materials[MaterialCount].Bytes[8] = 0
						Materials[MaterialCount].Bytes[9] = 0
						Materials[MaterialCount].Bytes[10] =0
						Materials[MaterialCount].Bytes[11] = 4
						Materials[MaterialCount].Bytes[12] = 0
						Materials[MaterialCount].Bytes[13] = 238
						Materials[MaterialCount].Bytes[14] = 162
						Materials[MaterialCount].Bytes[15] = 0
						Materials[MaterialCount].Bytes[16] = 255
						Materials[MaterialCount].Bytes[17] = 0
						Materials[MaterialCount].Bytes[18] = 0
						Materials[MaterialCount].Bytes[19] = 0
						Materials[MaterialCount].Bytes[20] = 0
						
						if sceneMaterials[i].DiffuseMap != undefined then
						(
							local index = finditem TextureFilenames sceneMaterials[i].DiffuseMap.Filename
							if index == 0 then
							(
								append TextureFilenames sceneMaterials[i].DiffuseMap.Filename
								print ("Found a texture to export: " + sceneMaterials[i].DiffuseMap.Filename)
								index = TextureFilenames.Count
							)
							
							Materials[MaterialCount].MapOne = (index - 1)
						)
						if sceneMaterials[i].OpacityMap != undefined then
						(
							Materials[MaterialCount].Bytes[4] = 1
						)			
						--Need to de-dup for this to work probably
			--			if sceneMaterials[i].ReflectionMap != undefined then
			--			(
			--				local index = finditem TextureFilenames sceneMaterials[i].ReflectionMap.Filename
			--				if index == 0 then
			--				(
			--					append TextureFilenames sceneMaterials[i].ReflectionMap.Filename
			--					print ("Found a texture to export: " + sceneMaterials[i].ReflectionMap.Filename)
			--					index = TextureFilenames.Count
			--				)
			--				Materials[MaterialCount].MapTwo = (index - 1)
			--			) 
						if sceneMaterials[i].SpecularMap != undefined then
						(
							local index = finditem TextureFilenames sceneMaterials[i].SpecularMap.Filename
							if index == 0 then
							(
								append TextureFilenames sceneMaterials[i].SpecularMap.Filename
								print ("Found a texture to export: " + sceneMaterials[i].SpecularMap.Filename)
								index = TextureFilenames.Count
							)
							Materials[MaterialCount].MapFour = (index - 1)
					)
				)			
			)
			
			
			for i = 1 to TextureFilenames.Count do
			(
				local split = filterString TextureFilenames[i] "\\"
				TextureFilenames[i] = split[ split.Count ]
			)


			-- We now have a list of all the textures in the scene - Hurrah
			
			--NOTE - WRITE BOTH FLAGS AS ZERO :D - Screw the optionals for now
			--THERE ARE NO ROOMS, NO CONTROL MESHES.
			
			Magic = 10
			TextureCount = TextureFilenames.Count
			GroupCount =  GetNodeCount rootNode -- Each node is a group, which we'll break down ourselves
			HasBones = 0
			UnknownOne = 0
			AnchorCount = 0
			HasBinding = 0
			UnknownTwo = 0
			FlagsOne = 0
			FlagsTwo = 194
			HasDeforms = 0
			TransformCount = 0
			RoomCount = 0
				
						
			-- WRITE THE HEADER DATA
			
			WriteLong stream Magic #unsigned
			WriteLong stream TextureCount #unsigned
			WriteLong stream MaterialCount #unsigned
			WriteLong stream GroupCount #unsigned
			
			WriteByte stream HasBones #unsigned
			WriteByte stream UnknownOne #unsigned
			WriteByte stream AnchorCount #unsigned
			WriteByte stream HasBinding #unsigned
			WriteByte stream UnknownTwo #unsigned
			WriteByte stream FlagsOne #unsigned
			WriteByte stream FlagsTwo #unsigned
			WriteByte stream HasDeforms #unsigned
			
			WriteLong stream 0
			WriteLong stream 0
			WriteLong stream 0
			
			-- WRITE THE TEXTURE FILENAMES
			
			for i = 1 to TextureCount do
			(
				WriteString stream TextureFileNames[i]
				-- Add zero padding
				for j = 1 to ( 31 - TextureFileNames[i].Count ) do
				(
					WriteByte stream 0
				)			
			)	
				
			for i = 1 to MaterialCount do
			(
				WriteByte stream Materials[i].MapOne
				WriteByte stream Materials[i].MapTwo
				WriteByte stream Materials[i].MapThree
				WriteByte stream Materials[i].MapFour
				
				for j = 1 to 20 do
				(
					WriteByte stream Materials[i].Bytes[j]
				)
			)	
				
			-- Write each group from each node recursively
			
			for c in rootNode.Children do
			(
				WriteNode stream c
			)

			print "Closing Stream"
			fclose stream		
		
		),
		

		fn LoadMesh filename=
		(			
			ResetMaxFile #noPrompt
			
			local stream = fopen filename "rb"
			
			local filenamepath = getFileNamePath filename
			
			-- Create layers to add geometry into
			
			MainLayer = LayerManager.GetLayer 0
			ControlLayer = LayerManager.NewLayerFromName "ControlLayer"
			RoomLayer = LayerManager.NewLayerFromName "RoomLayer"
			
	
			-- Read the Mesh Header
			Magic = ReadLong stream #unsigned
			TextureCount = ReadLong stream #unsigned
			MaterialCount = ReadLong stream #unsigned
			GroupCount = ReadLong stream #unsigned			
			HasBones = ReadByte stream #unsigned	
			
			UnknownOne = ReadByte stream #unsigned
	
			AnchorCount = ReadByte stream #unsigned			
			HasBinding = ReadByte stream #unsigned	
		
			UnknownTwo = ReadByte stream #unsigned
				
			FlagsOne = ReadByte stream #unsigned
			FlagsTwo = ReadByte stream #unsigned
			
			HasDeforms = ReadByte stream #unsigned
			
			-- End of Mesh Header
			
			-- Optionals
			
			-- Transforms ( Unknown )
			if (bit.and FlagsTwo 2) == 2 then
			(
				TransformCount = ReadLong stream #unsigned
			)
			else
			(
				TransformCount = 0
			)			
			
			-- Submesh Count
			if (bit.and FlagsTwo 128) == 128 then
			(
				SubmeshCount = ReadLong stream #unsigned
			)
			else
			(
				SubmeshCount = 0
			)
			
			-- Room Count
			if (bit.and FlagsTwo 64) == 64 then
			(
				RoomCount = ReadLong stream #unsigned
			)
			else
			(
				RoomCount = 0
			)
			
			-- Read in the Texture Filenames
			TextureFilenames = #()
			
			for i = 1 to TextureCount do
			(
				TextureFilenames[i] = ReadFixedString stream 32
				print ("found a texture to load: "  + TextureFilenames[i])
			)
						
			-- Read Materials in
			
			Materials = #()
			MaxMaterials = #()
		
			for i = 1 to MaterialCount do
			(
				Materials[i] = TMMaterial()
				Materials[i].Bytes = #()
				
				Materials[i].MapOne = ReadByte stream
				Materials[i].MapTwo = ReadByte stream
				Materials[i].MapThree = ReadByte stream
				Materials[i].MapFour = ReadByte stream
				
				for x = 1 to 20 do
				(
					Materials[i].Bytes[x] = ReadByte stream
				)
				
		
				MaxMaterials[i] = StandardMaterial()
				
				-- If diffuse is a texture
				if Materials[i].MapOne >= 0 then
				(
					local TexName = TextureFilenames [Materials[i].MapOne + 1]
					-- Arrays are 1 indexed, but these are zero indexed, so up it one
					MaxMaterials[i].DiffuseMap = BitmapTexture filename:(TexName)
					MaxMaterials[i].DiffuseMap.Name = TexName
					MaxMaterials[i].DiffuseMapEnable = true
					MaxMaterials[i].DiffuseMap.AlphaSource = 2
	   				ShowTextureMap MaxMaterials[i] MaxMaterials[i].DiffuseMap true
					
					--Choose whether to use alpha or not
					if Materials[i].Bytes[4] != 0 then
					(
						MaxMaterials[i].OpacityMap = BitmapTexture filename:(TexName)
						MaxMaterials[i].OpacityMap.Name = TexName
						MaxMaterials[i].OpacityMapEnable = true	
						MaxMaterials[i].OpacityMap.MonoOutput = 1 --make sure to use alpha
						ShowTextureMap MaxMaterials[i] MaxMaterials[i].OpacityMap true	
									
					)
				)
				--If Reflection is a texture
				if Materials[i].MapTwo >= 0 then
				(
					local TexName = TextureFilenames [Materials[i].MapTwo + 1]
					-- Arrays are 1 indexed, but these are zero indexed, so up it one
					MaxMaterials[i].ReflectionMap = BitmapTexture filename:(TexName)
					MaxMaterials[i].ReflectionMap.Name = TexName
					MaxMaterials[i].ReflectionMapEnable = true
					MaxMaterials[i].ReflectionMap.PremultAlpha = false				
	   				ShowTextureMap MaxMaterials[i] MaxMaterials[i].ReflectionMap true					
				)

				--If there is a lightmap, ignore it
			
				--If there is a specular map
				if Materials[i].MapFour >= 0 then
				(
					local TexName = TextureFilenames [Materials[i].MapFour + 1]
					-- Arrays are 1 indexed, but these are zero indexed, so up it one
					MaxMaterials[i].SpecularLevelMap = BitmapTexture filename:(TexName)
					MaxMaterials[i].SpecularLevelMap.Name = TexName
					MaxMaterials[i].SpecularLevelMapEnable = true
	   				ShowTextureMap MaxMaterials[i] MaxMaterials[i].SpecularLevelMap true					
				)	

			)
			
				
			-- Textures and Materials now loaded, with any luck
			
			ControlLayer.current = true
			
			-- Read in Optional Control Meshes
			
			if HasBinding != 0 then
			(
				LoadControlMesh stream "Binding"
			)
			
			if (bit.and FlagsTwo 1) == 1 then
			(
				LoadControlMesh stream "Floor"
			)
					
			if (bit.and FlagsTwo 8) == 8 then
			(
				LoadControlMesh stream "Wall"
			)
					
			if (bit.and FlagsOne 2) == 2 then
			(
				LoadControlMesh stream "Land"
			)
					
			if (bit.and FlagsTwo 4) == 4 then
			(
				LoadControlMesh stream "Mesh1"
			)
							
			if (bit.and FlagsOne 4) == 4 then
			(
				LoadControlMesh stream "Foundation"
			)
			
			if (bit.and FlagsTwo 16) == 16 then
			(
				LoadControlMesh stream "Mesh2"
			)
			
			
			RoomLayer.current = true		
			
			-- Read In Optional Rooms
			
			for i = 1 to RoomCount do
			(
				local name = ReadFixedString stream 32
				LoadControlMesh stream name
			)
			
			MainLayer.current = true
			
			-- Read In Groups
			
			for i = 1 to GroupCount do
			(			
				LoadGroup stream profilestream
				gc()
			)					
			
			fclose stream		
		)	
	)
	

	label lbl1 "http://tmws.themoviesplanet.com" pos:[46,268] width:236 height:17
	label lbl2 "http://www.ireallydontcare.com" pos:[46,282] width:159 height:16
	label lbl6 "http://www.themovieseditor.com" pos:[48,254] width:158 height:13
	label lbl7 "Version 0.3" pos:[7,10] width:162 height:17
	
	on btnImport pressed do
	(
		local importPly fname = getOpenFileName types:"msh (*.msh)|*.msh|All (*.*)|*.*|"
	
		if fname != undefined then
		(
			if DoesFileExist fname then	
			(
				local mesh = TMMesh()
				mesh.LoadMesh fname
				
				gc delayed:false
				max views redraw
	
			)		
		
		)

	)
	on btnExport pressed do
	(
		local importPly fname = GetSaveFileName types:"msh (*.msh)|*.msh|All (*.*)|*.*|"
		
		if fname != undefined then
		(	
			local mesh = TMMesh()
			mesh.SaveMesh fname	
		)
	)
)
if FloaterMshFloater != undefined do
(
		closerolloutfloater FloaterMshFloater
)		
FloaterMshFloater = newRolloutFloater "TM Mesh Handler" 250 350 
addRollout MeshRollout FloaterMshFloater

